# -*- coding: utf-8 -*-
"""
Created on Sat Sep 28 22:03:59 2024

@author: richie bao
"""
import mesa
from agents import Person, Tree, Patch
from model import PollutionModel

import matplotlib.colors as colors
import matplotlib.pyplot as plt
import matplotlib.cm as cm

_COLORS4Patch = {
    True: "Red",
    False: "white",
    # "Aqua",
    # "Blue",
    # "Fuchsia",
    # "Gray",
    # "Green",
    # "Lime",
    # "Maroon",
    # "Navy",
    # "Olive",
    # "Orange",
    # "Purple",
    # "Red",
    # "Silver",
    # "Teal",
    # "White",
    # "Yellow",
}

_COLORS4WorldStatus = {
    "Person": "#00AA00",
    "Tree": "#880000",
    "PollutionSum": "#000000",
}


def pollution_portrayal(agent):
    if agent is None:
        return

    if type(agent) is Patch:
        portrayal = {
            "Shape": "rect",
            "w": 1,
            "h": 1,
            "Filled": "true",
            "Layer": 0,
        }
        portrayal["x"] = agent.get_row()
        portrayal["y"] = agent.get_col()
        portrayal["Color"] = _COLORS4Patch[agent.get_state()]

        # norm = colors.Normalize(vmin=0, vmax=0.5)
        # cmap = cm.get_cmap('Greys')
        # rgba_color = cmap(norm(agent.get_state()))
        # portrayal["Color"] =colors.to_hex(rgba_color)
        # portrayal["Color"]="white"

    if type(agent) is Person:
        portrayal = {
            "Shape": "./shapes/person.png",
            "w": 1,
            "h": 1,
            "Filled": "true",
            "Layer": 1,
        }
        portrayal["Color"] = "Olive"

    if type(agent) is Tree:
        portrayal = {
            "Shape": "./shapes/tree.png",
            "w": 1,
            "h": 1,
            "Filled": "true",
            "Layer": 1,
        }
        portrayal["Color"] = "Green"

    return portrayal


worldStatus_chart = mesa.visualization.ChartModule(
    [
        {"Label": label, "Color": color}
        for (label, color) in _COLORS4WorldStatus.items()
    ]
)


model_params = {
    "title": mesa.visualization.StaticText(
        "Pollution: predator-prey ecosystem"
    ),
    "parameters": mesa.visualization.StaticText("Parameters:"),
    "initial_population": mesa.visualization.Slider(
        "initial_population", 30, 0, 100
    ),
    "birth_rate": mesa.visualization.Slider("birth rate", 0.1, 0, 0.2, 0.01),
    "planting_rate": mesa.visualization.Slider(
        "planting rate", 0.05, 0, 0.1, 0.01
    ),
    "power_plants": mesa.visualization.Slider("power plants", 2, 0, 20),
    "polluting_rate": mesa.visualization.Slider("polluting rate", 3, 0, 5),
}


canvas_element = mesa.visualization.CanvasGrid(
    pollution_portrayal, 50, 50, 862, 862
)

server = mesa.visualization.ModularServer(
    PollutionModel,
    [canvas_element, worldStatus_chart],
    "Pollution: predator-prey ecosystem",
    model_params,
)
server.port = 8523
server.launch(open_browser=True)
